'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _importFrom = require('import-from');

var _loaderWebpackConfig = require('./loader-webpack-config');

var _loaderWebpackConfig2 = _interopRequireDefault(_loaderWebpackConfig);

var _defaultWebpackConfig = require('./default-webpack-config');

var _defaultWebpackConfig2 = _interopRequireDefault(_defaultWebpackConfig);

var _importModule = require('react-cosmos-utils/lib/import-module');

var _importModule2 = _interopRequireDefault(_importModule);

var _reactCosmosConfig = require('react-cosmos-config');

var _reactCosmosConfig2 = _interopRequireDefault(_reactCosmosConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var moduleExists = function moduleExists(modulePath) {
  try {
    return modulePath && require.resolve(modulePath) && true;
  } catch (err) {
    return false;
  }
};

var exportPlaygroundFiles = function exportPlaygroundFiles(outputPath) {
  _fsExtra2.default.copySync(_path2.default.join(__dirname, 'static/favicon.ico'), outputPath + '/favicon.ico');

  _fsExtra2.default.copySync(require.resolve('react-cosmos-component-playground'), outputPath + '/bundle.js');

  var playgroundHtml = _fsExtra2.default.readFileSync(_path2.default.join(__dirname, 'static/index.html'), 'utf8');
  var playgroundOpts = JSON.stringify({
    loaderUri: './loader/index.html'
  });
  _fsExtra2.default.writeFileSync(outputPath + '/index.html', playgroundHtml.replace('__PLAYGROUND_OPTS__', playgroundOpts));
};

var runWebpackCompiler = function runWebpackCompiler(webpack, config) {
  return new Promise(function (resolve, reject) {
    var compiler = webpack(config);
    compiler.run(function (err, stats) {
      if (err) {
        reject(err);
      } else {
        resolve(stats);
      }
    });
  });
};

module.exports = function startExport(cosmosConfigPath) {
  var cosmosConfig = (0, _reactCosmosConfig2.default)(cosmosConfigPath);
  var webpackConfigPath = cosmosConfig.webpackConfigPath,
      outputPath = cosmosConfig.outputPath,
      publicPath = cosmosConfig.publicPath,
      publicUrl = cosmosConfig.publicUrl;


  var webpack = (0, _importFrom.silent)(cosmosConfigPath, 'webpack');
  if (!webpack) {
    console.warn('[Cosmos] webpack dependency missing!');
    console.log('Install using "yarn add webpack" or "npm install webpack"');
    return;
  }

  var userWebpackConfig = void 0;
  if (moduleExists(webpackConfigPath)) {
    console.log('[Cosmos] Using webpack config found at ' + webpackConfigPath);
    userWebpackConfig = (0, _importModule2.default)(require(webpackConfigPath));
  } else {
    console.log('[Cosmos] No webpack config found, using default config');
    userWebpackConfig = (0, _defaultWebpackConfig2.default)(cosmosConfigPath);
  }

  var loaderWebpackConfig = (0, _loaderWebpackConfig2.default)({
    webpack: webpack,
    userWebpackConfig: userWebpackConfig,
    cosmosConfigPath: cosmosConfigPath,
    shouldExport: true
  });

  // Copy static files first, so that the built index.html overrides the its
  // template file (in case the static assets are served from the root path)
  if (publicPath) {
    if (outputPath.indexOf(publicPath) === -1) {
      var exportPublicPath = _path2.default.join(outputPath, publicUrl);
      _fsExtra2.default.copySync(publicPath, exportPublicPath);
    } else {
      console.warn('[Cosmos] Warning: Can\'t export public path because it contains the export path! (avoiding infinite loop)');
      console.warn('Public path:', publicPath);
      console.warn('Export path:', outputPath);
    }
  }

  runWebpackCompiler(webpack, loaderWebpackConfig).then(function () {
    exportPlaygroundFiles(outputPath);
  }).then(function () {
    console.log('[Cosmos] Export Complete! Find the exported files here:');
    console.log(outputPath);
  }, function (err) {
    console.error('[Cosmos] Export Failed! See error below:');
    console.error(err);
  });
};