'use strict';

/* global window */

// This has to be done before React is imported. We do it before importing
// anything which might import React
// https://github.com/facebook/react-devtools/issues/76#issuecomment-128091900
if (process.env.NODE_ENV === 'development') {
  // Accessing the parent window can throw when loading a static export without
  // a web server (i.e. via file:/// protocol)
  try {
    window.__REACT_DEVTOOLS_GLOBAL_HOOK__ = window.parent.__REACT_DEVTOOLS_GLOBAL_HOOK__;
  } catch (err) {
    console.error(err);
  }
}

var _require = require('react-cosmos-utils/lib/import-module'),
    importModule = _require.default;

var _require2 = require('./normalize-modules'),
    normalizeComponents = _require2.normalizeComponents,
    normalizeFixtures = _require2.normalizeFixtures;

var _require3 = require('react-cosmos-loader'),
    mount = _require3.mount;

// eslint-disable-next-line no-undef


var _COSMOS_CONFIG = COSMOS_CONFIG,
    containerQuerySelector = _COSMOS_CONFIG.containerQuerySelector;


var start = function start() {
  // Module is imported whenever this function is called, making sure the
  // lastest module version is used after a HMR update
  var getUserModules = require('./user-modules').default;

  var _getUserModules = getUserModules(),
      components = _getUserModules.components,
      fixtures = _getUserModules.fixtures,
      proxies = _getUserModules.proxies;

  mount({
    proxies: importModule(proxies),
    components: normalizeComponents(components),
    fixtures: normalizeFixtures(fixtures),
    containerQuerySelector: containerQuerySelector
  });
};

start();

if (module.hot) {
  module.hot.accept('./user-modules', function () {
    start();
  });
}

// Hook for Cypress to simulate a HMR update
window.__startCosmosLoader = start;