'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _importFrom = require('import-from');

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _webpackDevMiddleware = require('webpack-dev-middleware');

var _webpackDevMiddleware2 = _interopRequireDefault(_webpackDevMiddleware);

var _webpackHotMiddleware = require('webpack-hot-middleware');

var _webpackHotMiddleware2 = _interopRequireDefault(_webpackHotMiddleware);

var _loaderWebpackConfig = require('./loader-webpack-config');

var _loaderWebpackConfig2 = _interopRequireDefault(_loaderWebpackConfig);

var _defaultWebpackConfig = require('./default-webpack-config');

var _defaultWebpackConfig2 = _interopRequireDefault(_defaultWebpackConfig);

var _importModule = require('react-cosmos-utils/lib/import-module');

var _importModule2 = _interopRequireDefault(_importModule);

var _moduleExists = require('react-cosmos-utils/lib/module-exists');

var _moduleExists2 = _interopRequireDefault(_moduleExists);

var _reactCosmosConfig = require('react-cosmos-config');

var _reactCosmosConfig2 = _interopRequireDefault(_reactCosmosConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getPublicPath = function getPublicPath(cosmosConfig, userWebpackConfig) {
  return cosmosConfig.publicPath || userWebpackConfig.devServer && userWebpackConfig.devServer.contentBase;
};

module.exports = function startServer(cosmosConfigPath) {
  var cosmosConfig = (0, _reactCosmosConfig2.default)(cosmosConfigPath);
  var hostname = cosmosConfig.hostname,
      hot = cosmosConfig.hot,
      port = cosmosConfig.port,
      webpackConfigPath = cosmosConfig.webpackConfigPath,
      publicUrl = cosmosConfig.publicUrl;


  var webpack = (0, _importFrom.silent)(cosmosConfigPath, 'webpack');
  if (!webpack) {
    console.warn('[Cosmos] webpack dependency missing!');
    console.log('Install using "yarn add webpack" or "npm install webpack"');
    return;
  }

  var userWebpackConfig = void 0;
  if ((0, _moduleExists2.default)(webpackConfigPath)) {
    console.log('[Cosmos] Using webpack config found at ' + webpackConfigPath);
    userWebpackConfig = (0, _importModule2.default)(require(webpackConfigPath));
  } else {
    console.log('[Cosmos] No webpack config found, using default config');
    userWebpackConfig = (0, _defaultWebpackConfig2.default)(cosmosConfigPath);
  }

  if (cosmosConfig.proxies) {
    console.warn('[Cosmos] Warning: config.proxies is deprecated!');
    console.warn('Please check latest proxy docs: https://github.com/react-cosmos/react-cosmos#proxies');
  }

  var loaderWebpackConfig = (0, _loaderWebpackConfig2.default)({
    webpack: webpack,
    userWebpackConfig: userWebpackConfig,
    cosmosConfigPath: cosmosConfigPath
  });
  var loaderCompiler = webpack(loaderWebpackConfig);
  var app = (0, _express2.default)();

  app.use((0, _webpackDevMiddleware2.default)(loaderCompiler, {
    publicPath: '/loader/',
    noInfo: true
  }));

  if (hot) {
    app.use((0, _webpackHotMiddleware2.default)(loaderCompiler));
  }

  var publicPath = getPublicPath(cosmosConfig, userWebpackConfig);
  if (publicPath) {
    console.log('[Cosmos] Serving static files from ' + publicPath);
    app.use(publicUrl, _express2.default.static(publicPath));
  }

  var playgroundHtml = _fs2.default.readFileSync(_path2.default.join(__dirname, 'static/index.html'), 'utf8');
  var playgroundOpts = JSON.stringify({
    loaderUri: './loader/index.html'
  });

  app.get('/', function (req, res) {
    res.send(playgroundHtml.replace('__PLAYGROUND_OPTS__', playgroundOpts));
  });

  app.get('/bundle.js', function (req, res) {
    res.sendFile(require.resolve('react-cosmos-component-playground'));
  });

  app.get('/favicon.ico', function (req, res) {
    res.sendFile(_path2.default.join(__dirname, 'static/favicon.ico'));
  });

  app.listen(port, hostname, function (err) {
    if (err) {
      throw err;
    }
    console.log('[Cosmos] See you at http://' + hostname + ':' + port + '/');
  });
};